
# This file was generated by 'versioneer.py' (0.19) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

import json

version_json = '''
{
 "date": "2025-04-16T11:42:20+0200",
 "dirty": true,
 "error": null,
 "full-revisionid": "943a31433f896fe25cd9884d836cc029465c1e3b",
 "version": "2.59.1"
}
'''  # END VERSION_JSON


def get_versions():
    return json.loads(version_json)
