(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[5822],{87321:function(e,s,t){(window.__NEXT_P=window.__NEXT_P||[]).push(["/add-systems/multiple",function(){return t(17035)}])},17035:function(e,s,t){"use strict";t.r(s);var o=t(24246),n=t(77181),r=t(77213),d=t(77830),i=t(58754),a=t(15826);s.default=()=>(0,o.jsxs)(r.Z,{title:"Choose vendors",children:[(0,o.jsx)(i.Z,{heading:"Add systems",breadcrumbItems:[{title:"Add systems",href:d.xo},{title:"Choose vendors"}]}),(0,o.jsx)(n.xuv,{w:{base:"100%",md:"75%"},children:(0,o.jsxs)(n.xvT,{fontSize:"sm",mb:8,children:["Select your vendors below and they will be added as systems to your data map. Fides Compass will automatically populate the system information so that you can quickly configure privacy requests and consent. To add custom systems or unlisted vendors, please",(0,o.jsxs)(n.rUS,{href:d.N5,color:"complimentary.500",children:[" ","click here."," "]})]})}),(0,o.jsx)(a.$,{redirectRoute:d.oG})]})}},function(e){e.O(0,[8033,6060,4481,401,8934,5826,2888,9774,179],function(){return e(e.s=87321)}),_N_E=e.O()}]);