.ActivityTimelineList_itemButton___zZAL{display:block;width:100%;border:1px solid transparent;border-radius:6px;transition:border-color .2s ease-in-out;margin-bottom:20px;padding:12px 20px}.ActivityTimelineList_itemButton--error__s_p3y,.ActivityTimelineList_itemButton___zZAL:hover{border-color:var(--fidesui-neutral-100)}.ActivityTimelineList_itemButton___zZAL:focus-visible{border-color:var(--fidesui-neutral-700)}.ActivityTimelineList_itemButton--error__s_p3y,.ActivityTimelineList_itemButton--error__s_p3y:focus,.ActivityTimelineList_itemButton--error__s_p3y:hover{border-left:8px solid var(--fidesui-error)}.ActivityTimelineList_header__c6_1X{cursor:pointer;width:100%;display:flex;flex-wrap:wrap;gap:8px;align-items:center}.ActivityTimelineList_title__BhPbu{font-weight:600}.ActivityTimelineList_title--error__OWYRS{color:var(--fidesui-error)}.ActivityTimelineList_timestamp__kg6oj{color:var(--fidesui-neutral-700)}.ActivityTimelineList_viewLogs__dD3WJ{color:var(--fidesui-link)}.ActivityTimelineList_logs__uuT2f{height:0;overflow:hidden;transition:height .2s ease-in-out;box-sizing:border-box;padding:0 20px}.ActivityTimelineList_logs--open__rkg5L{height:auto;margin-top:20px;margin-bottom:20px}.RequestDetailsRow_value__WMXn3{color:var(--fidesui-minos)}.RequestDetailsRow_label__dn3CQ{color:var(--fidesui-neutral-700)}