<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <title>Privacy Request Complete</title>
</head>
<body>
<main>
    <p>
        Your privacy request for deletion has been completed.
    </p>
</main>
</body>
</html>