from fides.api.service.connectors.postgres_connector import PostgreSQLConnector


class TimescaleConnector(PostgreSQLConnector):
    """TimescaleDB Connector - This currently supports identical functionality to the PostgreSQL connector"""
