(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[1571],{11207:function(r,e,n){(window.__NEXT_P=window.__NEXT_P||[]).push(["/data-discovery/discovery/[resourceUrn]",function(){return n(55124)}])},55124:function(r,e,n){"use strict";n.r(e);var s=n(24246);n(27378);var c=n(35287),t=n(77830),i=n(58754),u=n(69828),o=n(47182),a=n(11904);e.default=()=>{let{resourceUrn:r,navigateToDiscoveryResults:e}=(0,o.Z)();return(0,s.jsxs)(c.Z,{title:"Data discovery",children:[(0,s.jsx)(i.Z,{heading:"Data discovery",children:(0,s.jsx)(u.Z,{parentLink:t.zE,resourceUrn:r,onPathClick:r=>e({resourceUrn:r})})}),(0,s.jsx)(a.Z,{resourceUrn:r})]})}}},function(r){r.O(0,[8033,6060,2858,2866,9278,4481,6395,6372,3702,1904,2888,9774,179],function(){return r(r.s=11207)}),_N_E=r.O()}]);